/**
 * MCS Media Computer Software
 * Copyright 2018 by Wilfried Klaas
 * Project: SPSEmulator
 * File: Not.java
 * EMail: W.Klaas@gmx.de
 * Created: 25.11.2018 wklaa_000
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>
 */
package de.mcs.tools.sps.mnemonic;

import org.apache.commons.lang3.StringUtils;

import de.mcs.tools.sps.exceptions.IllegalArgument;
import de.mcs.tools.sps.exceptions.SyntaxError;

/**
 * DIN, DIN #x: Input from digital input to register A. If no #x given, all 4
 * inputs are shown in the register , otherwise the lowest bit of A will show
 * the input value #x. #x in range 1..4
 * 
 * @author wklaa_000
 *
 */
public class DIN extends AbstractMnemonic implements Mnemonic {

  byte value;

  public DIN(String line) throws SyntaxError {
    super(line);
    value = 0;
  }

  @Override
  public void checkArgument() throws SyntaxError {
    if (StringUtils.isEmpty(getArgument())) {
      value = 0;
    } else {
      int myValue = getArgumentAsNumber();
      if ((myValue < 1) || (myValue > 4)) {
        throw new IllegalArgument(getLineNumber(),
            String.format("argument %s is not in range 1..4 for %s.", getArgument(), this.getClass().getSimpleName()));
      }
      value = (byte) (myValue & 0xFF);
    }
  }

  @Override
  public int getByte() {
    return 0x64 + value;
  }

  @Override
  public boolean isUsingPage() {
    return false;
  }

  @Override
  public boolean hasArgument() {
    return true;
  }

}
