/**
 * MCS Media Computer Software
 * Copyright 2018 by Wilfried Klaas
 * Project: SPSEmulator
 * File: Not.java
 * EMail: W.Klaas@gmx.de
 * Created: 25.11.2018 wklaa_000
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>
 */
package de.mcs.tools.sps.mnemonic;

import org.apache.commons.lang3.StringUtils;

import de.mcs.tools.sps.exceptions.IllegalArgument;
import de.mcs.tools.sps.exceptions.SyntaxError;

/**
 * DOUT, DOUT #x: Output of register A. If no #x given, register a is output
 * fully, otherwise the lowest bit of A will be shon on output #x. #x in range
 * 1..4
 * 
 * @author wklaa_000
 *
 */
public class DOUT extends AbstractMnemonic implements Mnemonic {

  static int PORT = 0x10;
  static int DOUT = 0x54;
  byte value;
  boolean isPort;

  public DOUT(String line) throws SyntaxError {
    super(line);
    value = 0;
    isPort = false;
  }

  @Override
  public void checkArgument() throws SyntaxError {
    String argument = getArgument();
    if (StringUtils.isEmpty(argument)) {
      value = 0;
    } else if (argument.startsWith("#")) {
      isPort = true;
      String myArgument = argument.substring(1);
      int myValue = getArgumentAsNumber(myArgument);
      if ((myValue < 0) || (myValue > 15)) {
        throw new IllegalArgument(getLineNumber(),
            String.format("argument %s is not in range 1..4 for %s.", argument, this.getClass().getSimpleName()));
      }
      value = (byte) (myValue & 0xFF);
    } else {
      int myValue = getArgumentAsNumber();
      if ((myValue < 1) || (myValue > 4)) {
        throw new IllegalArgument(getLineNumber(),
            String.format("argument %s is not in range 1..4 for %s.", argument, this.getClass().getSimpleName()));
      }
      value = (byte) (myValue & 0xFF);
    }
  }

  @Override
  public int getByte() {
    if (isPort) {
      return PORT + value;
    }
    return DOUT + value;
  }

  @Override
  public boolean isUsingPage() {
    return false;
  }

  @Override
  public boolean hasArgument() {
    return true;
  }

}
