/**
 * MCS Media Computer Software
 * Copyright 2018 by Wilfried Klaas
 * Project: SPSEmulator
 * File: Mnemonic.java
 * EMail: W.Klaas@gmx.de
 * Created: 25.11.2018 wklaa_000
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>
 */
package de.mcs.tools.sps.mnemonic;

import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;

import de.mcs.tools.sps.exceptions.SyntaxError;

/**
 * @author wklaa_000
 *
 */
public class MnemonicFactory {

  public MnemonicFactory() {
  }

  public static Mnemonic getMnemonic(String line, int lineNumber) throws SyntaxError {
    String name = AbstractMnemonic.getName(line);
    try {
      Class<Mnemonic> mnemonicClass = (Class<Mnemonic>) Class.forName("de.mcs.tools.sps.mnemonic." + name);
      Constructor<Mnemonic> constructor = mnemonicClass.getConstructor(String.class);
      Mnemonic newInstance = constructor.newInstance(line);
      newInstance.setLineNumber(lineNumber);
      newInstance.checkArgument();
      return newInstance;
    } catch (SyntaxError e) {
      throw e;
    } catch (ClassNotFoundException e) {
      throw new SyntaxError(String.format("syntax error in line %d. Illegal Mnemonic %s", lineNumber, name));
    } catch (SecurityException e) {
      e.printStackTrace();
    } catch (NoSuchMethodException e) {
      e.printStackTrace();
    } catch (InstantiationException e) {
      e.printStackTrace();
    } catch (IllegalAccessException e) {
      e.printStackTrace();
    } catch (IllegalArgumentException e) {
      e.printStackTrace();
    } catch (InvocationTargetException e) {
      e.printStackTrace();
    }
    return null;
  }

}
