package de.mcs.tools.sps.mnemonic;

import static org.junit.Assert.assertFalse;
import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertTrue;

import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.Test;

import de.mcs.tools.sps.exceptions.IllegalArgument;
import de.mcs.tools.sps.exceptions.SyntaxError;

class TestDOUT {

  private static final int BYTE_VALUE = 0x54;
  private static final int PORT_BYTE_VALUE = 0x10;
  private final String MNEMONIC = "DOUT";

  @Test
  void testMnemonic() throws SyntaxError {
    DOUT mnemonic = new DOUT(MNEMONIC);
    mnemonic.checkArgument();
    assertEquals(BYTE_VALUE, mnemonic.getByte());
    assertTrue(mnemonic.hasArgument());
    assertFalse(mnemonic.isUsingPage());

    mnemonic = new DOUT(MNEMONIC + " 0x01");
    mnemonic.checkArgument();
    assertEquals(BYTE_VALUE + 1, mnemonic.getByte());

    mnemonic = new DOUT(MNEMONIC + " 0x04");
    mnemonic.checkArgument();
    assertEquals(BYTE_VALUE + 0x04, mnemonic.getByte());

    mnemonic = new DOUT(MNEMONIC + " #0x04");
    mnemonic.checkArgument();
    assertEquals(PORT_BYTE_VALUE + 0x04, mnemonic.getByte());

    mnemonic = new DOUT(MNEMONIC + " #0x00");
    mnemonic.checkArgument();
    assertEquals(PORT_BYTE_VALUE + 0x00, mnemonic.getByte());

    mnemonic = new DOUT(MNEMONIC + " #0x0f");
    mnemonic.checkArgument();
    assertEquals(PORT_BYTE_VALUE + 0x0f, mnemonic.getByte());

    Assertions.assertThrows(SyntaxError.class, () -> {
      DOUT mno1 = new DOUT(MNEMONIC + " 0x05");
      mno1.checkArgument();
    });

    Assertions.assertThrows(IllegalArgument.class, () -> {
      DOUT mno1 = new DOUT(MNEMONIC + " 0x10");
      mno1.checkArgument();
    });

    Assertions.assertThrows(IllegalArgument.class, () -> {
      DOUT mno1 = new DOUT(MNEMONIC + " #0x10");
      mno1.checkArgument();
    });
  }

  @Test
  void testMnemonicFactory() throws SyntaxError {
    Mnemonic mnemonic = MnemonicFactory.getMnemonic(MNEMONIC, 0);
    assertEquals(DOUT.class, mnemonic.getClass());

    mnemonic = MnemonicFactory.getMnemonic(MNEMONIC + " #0x04", 0);
    assertEquals(DOUT.class, mnemonic.getClass());

    Assertions.assertThrows(SyntaxError.class, () -> {
      MnemonicFactory.getMnemonic(MNEMONIC + " 0x00", 0);
    });

    Assertions.assertThrows(SyntaxError.class, () -> {
      MnemonicFactory.getMnemonic(MNEMONIC + " 5", 0);
    });

    mnemonic = MnemonicFactory.getMnemonic(MNEMONIC + " 0x02", 0);
    assertEquals(BYTE_VALUE + 0x02, mnemonic.getByte());

    mnemonic = MnemonicFactory.getMnemonic(MNEMONIC + " #3", 0);
    assertEquals(PORT_BYTE_VALUE + 3, mnemonic.getByte());

    mnemonic = MnemonicFactory.getMnemonic(MNEMONIC + " 0b00000100", 0);
    assertEquals(BYTE_VALUE + 4, mnemonic.getByte());

  }
}
