package de.mcs.tools.sps.mnemonic;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertFalse;
import static org.junit.jupiter.api.Assertions.assertTrue;

import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.Test;

import de.mcs.tools.sps.exceptions.IllegalArgument;
import de.mcs.tools.sps.exceptions.SyntaxError;

class TestLDA {

  private static final int BYTE_VALUE = 0x40;
  private static final int BYTE_VALUE_DIN = 0x64;
  private static final int BYTE_VALUE_ADC = 0x69;
  private static final int BYTE_VALUE_RC = 0x6B;
  private final String MNEMONIC = "LDA";

  @Test
  void testMnemonic() throws SyntaxError {
    LDA mnemonic = new LDA(MNEMONIC + " #0x00");
    mnemonic.checkArgument();
    assertEquals(BYTE_VALUE, mnemonic.getByte());
    assertTrue(mnemonic.hasArgument());
    assertFalse(mnemonic.isUsingPage());

    mnemonic = new LDA(MNEMONIC + " #0x01");
    mnemonic.checkArgument();
    assertEquals(BYTE_VALUE + 0x01, mnemonic.getByte());

    mnemonic = new LDA(MNEMONIC + " #0x0f");
    mnemonic.checkArgument();
    assertEquals(BYTE_VALUE + 0x0f, mnemonic.getByte());

    Assertions.assertThrows(SyntaxError.class, () -> {
      LDA mnemonic1 = new LDA(MNEMONIC);
      mnemonic1.checkArgument();
    });

    Assertions.assertThrows(IllegalArgument.class, () -> {
      LDA mnemonic1 = new LDA(MNEMONIC + " 0x01");
      mnemonic1.checkArgument();
    });

    Assertions.assertThrows(IllegalArgument.class, () -> {
      LDA mnemonic1 = new LDA(MNEMONIC + " #0x10");
      mnemonic1.checkArgument();
    });
  }

  @Test
  void testMnemonicDigital() throws SyntaxError {
    LDA mnemonic = new LDA(MNEMONIC + " DIN");
    mnemonic.checkArgument();
    assertEquals(BYTE_VALUE_DIN, mnemonic.getByte());
    assertTrue(mnemonic.hasArgument());
    assertFalse(mnemonic.isUsingPage());

    mnemonic = new LDA(MNEMONIC + " DIN1");
    mnemonic.checkArgument();
    assertEquals(BYTE_VALUE_DIN + 0x01, mnemonic.getByte());

    mnemonic = new LDA(MNEMONIC + " DIN4");
    mnemonic.checkArgument();
    assertEquals(BYTE_VALUE_DIN + 0x04, mnemonic.getByte());

    Assertions.assertThrows(SyntaxError.class, () -> {
      LDA mnemonic1 = new LDA(MNEMONIC + " DOUT");
      mnemonic1.checkArgument();
    });

    Assertions.assertThrows(IllegalArgument.class, () -> {
      LDA mnemonic1 = new LDA(MNEMONIC + " DIN5");
      mnemonic1.checkArgument();
    });
  }

  @Test
  void testMnemonicAnalog() throws SyntaxError {
    LDA mnemonic = new LDA(MNEMONIC + " ADC1");
    mnemonic.checkArgument();
    assertEquals(BYTE_VALUE_ADC, mnemonic.getByte());
    assertTrue(mnemonic.hasArgument());
    assertFalse(mnemonic.isUsingPage());

    mnemonic = new LDA(MNEMONIC + " ADC2");
    mnemonic.checkArgument();
    assertEquals(BYTE_VALUE_ADC + 0x01, mnemonic.getByte());

    Assertions.assertThrows(SyntaxError.class, () -> {
      LDA mnemonic1 = new LDA(MNEMONIC + " ADC");
      mnemonic1.checkArgument();
    });

    Assertions.assertThrows(SyntaxError.class, () -> {
      LDA mnemonic1 = new LDA(MNEMONIC + " ADC0");
      mnemonic1.checkArgument();
    });

    Assertions.assertThrows(SyntaxError.class, () -> {
      LDA mnemonic1 = new LDA(MNEMONIC + " ADC3");
      mnemonic1.checkArgument();
    });
  }

  @Test
  void testMnemonicReceiver() throws SyntaxError {
    LDA mnemonic = new LDA(MNEMONIC + " RC1");
    mnemonic.checkArgument();
    assertEquals(BYTE_VALUE_RC, mnemonic.getByte());
    assertTrue(mnemonic.hasArgument());
    assertFalse(mnemonic.isUsingPage());

    mnemonic = new LDA(MNEMONIC + " RC2");
    mnemonic.checkArgument();
    assertEquals(BYTE_VALUE_RC + 0x01, mnemonic.getByte());

    Assertions.assertThrows(SyntaxError.class, () -> {
      LDA mnemonic1 = new LDA(MNEMONIC + " RC");
      mnemonic1.checkArgument();
    });

    Assertions.assertThrows(SyntaxError.class, () -> {
      LDA mnemonic1 = new LDA(MNEMONIC + " RC0");
      mnemonic1.checkArgument();
    });

    Assertions.assertThrows(SyntaxError.class, () -> {
      LDA mnemonic1 = new LDA(MNEMONIC + " RC3");
      mnemonic1.checkArgument();
    });
  }

  @Test
  void testMnemonicFactory() throws SyntaxError {
    Mnemonic mnemonic = MnemonicFactory.getMnemonic(MNEMONIC + " #0x00", 0);
    assertEquals(LDA.class, mnemonic.getClass());

    mnemonic = MnemonicFactory.getMnemonic(MNEMONIC + " DIN", 0);
    assertEquals(LDA.class, mnemonic.getClass());

    mnemonic = MnemonicFactory.getMnemonic(MNEMONIC + " ADC1", 0);
    assertEquals(LDA.class, mnemonic.getClass());

    mnemonic = MnemonicFactory.getMnemonic(MNEMONIC + " RC2", 0);
    assertEquals(LDA.class, mnemonic.getClass());

    Assertions.assertThrows(SyntaxError.class, () -> {
      MnemonicFactory.getMnemonic(MNEMONIC, 0);
    });

    mnemonic = MnemonicFactory.getMnemonic(MNEMONIC + " #0x02", 0);
    assertEquals(BYTE_VALUE + 0x02, mnemonic.getByte());

    mnemonic = MnemonicFactory.getMnemonic(MNEMONIC + " #12", 0);
    assertEquals(BYTE_VALUE + 12, mnemonic.getByte());

    mnemonic = MnemonicFactory.getMnemonic(MNEMONIC + " #0b00001010", 0);
    assertEquals(BYTE_VALUE + 10, mnemonic.getByte());

  }
}
